@@ -7,17 +7,14 @@ import weakref
 
 from bokeh.layouts import row, column
 from bokeh.models import ( ColumnDataSource, Plot, DataRange1d, LinearAxis,
-        HoverTool, BoxZoomTool, ResetTool, PanTool, WheelZoomTool, Title,
-        Range1d, Quad, Text, value, BoxSelectTool, TapTool, Circle, OpenURL,)
-
-from bokeh.models.widgets import (DataTable, TableColumn, NumberFormatter,
-        Button, Select)
+        HoverTool, BoxZoomTool, ResetTool, PanTool, WheelZoomTool, Range1d,
+        Quad, TapTool, OpenURL, Button, Select)
 from bokeh.palettes import Spectral9
 from bokeh.plotting import figure
 from tornado import gen
 
 from ..config import config
-from ..diagnostics.progress_stream import progress_quads, nbytes_bar
+from ..diagnostics.progress_stream import nbytes_bar
 from .. import profile
 from ..utils import log_errors
 
@@ -104,7 +101,7 @@ class TaskStream(DashboardComponent):
         self.root.add_tools(
             hover, tap,
             BoxZoomTool(),
-            ResetTool(reset_size=False),
+            ResetTool(),
             PanTool(dimensions="width"),
             WheelZoomTool(dimensions="width")
         )
@@ -141,90 +138,6 @@ class TaskStream(DashboardComponent):
             self.source.stream(rectangles, self.n_rectangles)
 
 
-class TaskProgress(DashboardComponent):
-    """ Progress bars per task type """
-
-    def __init__(self, **kwargs):
-        data = progress_quads(dict(all={}, memory={}, erred={}, released={}))
-        self.source = ColumnDataSource(data=data)
-
-        x_range = DataRange1d()
-        y_range = Range1d(-8, 0)
-
-        self.root = Plot(
-            id='bk-task-progress-plot',
-            x_range=x_range, y_range=y_range, toolbar_location=None, **kwargs
-        )
-        self.root.add_glyph(
-            self.source,
-            Quad(top='top', bottom='bottom', left='left', right='right',
-                 fill_color="#aaaaaa", line_color="#aaaaaa", fill_alpha=0.2)
-        )
-        self.root.add_glyph(
-            self.source,
-            Quad(top='top', bottom='bottom', left='left', right='released-loc',
-                 fill_color="color", line_color="color", fill_alpha=0.6)
-        )
-        self.root.add_glyph(
-            self.source,
-            Quad(top='top', bottom='bottom', left='released-loc',
-                 right='memory-loc', fill_color="color", line_color="color",
-                 fill_alpha=1.0)
-        )
-        self.root.add_glyph(
-            self.source,
-            Quad(top='top', bottom='bottom', left='erred-loc',
-                 right='erred-loc', fill_color='#000000', line_color='#000000',
-                 fill_alpha=0.3)
-        )
-        self.root.add_glyph(
-            self.source,
-            Text(text='show-name', y='bottom', x='left', x_offset=5,
-                 text_font_size=value('10pt'))
-        )
-        self.root.add_glyph(
-            self.source,
-            Text(text='done', y='bottom', x='right', x_offset=-5,
-                 text_align='right', text_font_size=value('10pt'))
-        )
-
-        hover = HoverTool(
-            point_policy="follow_mouse",
-            tooltips="""
-                <div>
-                    <span style="font-size: 14px; font-weight: bold;">Name:</span>&nbsp;
-                    <span style="font-size: 10px; font-family: Monaco, monospace;">@name</span>
-                </div>
-                <div>
-                    <span style="font-size: 14px; font-weight: bold;">All:</span>&nbsp;
-                    <span style="font-size: 10px; font-family: Monaco, monospace;">@all</span>
-                </div>
-                <div>
-                    <span style="font-size: 14px; font-weight: bold;">Memory:</span>&nbsp;
-                    <span style="font-size: 10px; font-family: Monaco, monospace;">@memory</span>
-                </div>
-                <div>
-                    <span style="font-size: 14px; font-weight: bold;">Erred:</span>&nbsp;
-                    <span style="font-size: 10px; font-family: Monaco, monospace;">@erred</span>
-                </div>
-                """
-        )
-        self.root.add_tools(hover)
-
-    def update(self, messages):
-        with log_errors():
-            msg = messages['progress']
-            if not msg:
-                return
-            d = progress_quads(msg)
-            self.source.data.update(d)
-            if messages['tasks']['deque']:
-                self.root.title.text = ("Progress -- total: %(total)s, "
-                                        "in-memory: %(in-memory)s, processing: %(processing)s, "
-                                        "waiting: %(waiting)s, failed: %(failed)s"
-                                        % messages['tasks']['deque'][-1])
-
-
 class MemoryUsage(DashboardComponent):
     """ The memory usage across the cluster, grouped by task type """
 
@@ -278,100 +191,6 @@ class MemoryUsage(DashboardComponent):
                 "Memory Use: %0.2f MB" % (sum(msg['nbytes'].values()) / 1e6)
 
 
-class WorkerTable(DashboardComponent):
-    """ Status of the current workers
-
-    This is two plots, a text-based table for each host and a thin horizontal
-    plot laying out hosts by their current memory use.
-    """
-
-    def __init__(self, **kwargs):
-        names = ['processes', 'disk-read', 'cores', 'cpu', 'disk-write',
-                 'memory', 'last-seen', 'memory_percent', 'host']
-        self.source = ColumnDataSource({k: [] for k in names})
-
-        columns = {name: TableColumn(field=name,
-                                     title=name.replace('_percent', ' %'))
-                   for name in names}
-
-        cnames = ['host', 'cores', 'processes', 'memory', 'cpu', 'memory_percent']
-
-        formatters = {'cpu': NumberFormatter(format='0.0 %'),
-                      'memory_percent': NumberFormatter(format='0.0 %'),
-                      'memory': NumberFormatter(format='0 b'),
-                      'latency': NumberFormatter(format='0.00000'),
-                      'last-seen': NumberFormatter(format='0.000'),
-                      'disk-read': NumberFormatter(format='0 b'),
-                      'disk-write': NumberFormatter(format='0 b'),
-                      'net-send': NumberFormatter(format='0 b'),
-                      'net-recv': NumberFormatter(format='0 b')}
-
-        table = DataTable(
-            source=self.source, columns=[columns[n] for n in cnames],
-        )
-
-        for name in cnames:
-            if name in formatters:
-                table.columns[cnames.index(name)].formatter = formatters[name]
-
-        mem_plot = Plot(
-            title=Title(text="Memory Usage (%)"), toolbar_location=None,
-            x_range=Range1d(start=0, end=1), y_range=Range1d(start=-0.1, end=0.1),
-            **kwargs
-        )
-
-        mem_plot.add_glyph(
-            self.source,
-            Circle(x='memory_percent', y=0, size=10, fill_alpha=0.5)
-        )
-
-        mem_plot.add_layout(LinearAxis(), 'below')
-
-        hover = HoverTool(
-            point_policy="follow_mouse",
-            tooltips="""
-                <div>
-                  <span style="font-size: 10px; font-family: Monaco, monospace;">@host: </span>
-                  <span style="font-size: 10px; font-family: Monaco, monospace;">@memory_percent</span>
-                </div>
-                """
-        )
-        mem_plot.add_tools(hover, BoxSelectTool())
-
-        if 'sizing_mode' in kwargs:
-            sizing_mode = {'sizing_mode': kwargs['sizing_mode']}
-        else:
-            sizing_mode = {}
-
-        self.root = column(mem_plot, table, id='bk-worker-table', **sizing_mode)
-
-    def update(self, messages):
-        with log_errors():
-            try:
-                d = messages['workers']['deque'][-1]
-            except IndexError:
-                return
-
-            workers = sorted(d)
-
-            data = {}
-            data['host'] = workers
-            for name in ['cores', 'cpu', 'memory_percent', 'latency', 'last-seen',
-                         'memory', 'disk-read', 'disk-write', 'net-send',
-                         'net-recv']:
-                try:
-                    if name in ('cpu', 'memory_percent'):
-                        data[name] = [d[w][name] / 100 for w in workers]
-                    else:
-                        data[name] = [d[w][name] for w in workers]
-                except KeyError:
-                    pass
-
-            data['processing'] = [sorted(d[w]['processing']) for w in workers]
-            data['processes'] = [len(d[w]['addresses']) for w in workers]
-            self.source.data.update(data)
-
-
 class Processing(DashboardComponent):
     """ Processing and distribution per core
 
