@@ -19,7 +19,7 @@ from bokeh.models.widgets import DataTable, TableColumn
 from bokeh.plotting import figure
 from bokeh.palettes import Viridis11
 from bokeh.io import curdoc
-from toolz import pipe
+from toolz import pipe, merge
 try:
     import numpy as np
 except ImportError:
@@ -53,8 +53,7 @@ with open(os.path.join(os.path.dirname(__file__), 'template.html')) as f:
 
 template = jinja2.Template(template_source)
 
-template_variables = {'pages': ['status', 'workers', 'tasks', 'system',
-                                'profile', 'counters']}
+template_variables = {'pages': ['status', 'workers', 'tasks', 'system', 'profile']}
 
 
 def update(source, data):
@@ -87,39 +86,6 @@ def update(source, data):
         source.data.update(d)
 
 
-class StateTable(DashboardComponent):
-    """ Currently running tasks """
-
-    def __init__(self, scheduler):
-        self.scheduler = scheduler
-
-        names = ['Tasks', 'Stored', 'Processing', 'Waiting', 'No Worker',
-                 'Erred', 'Released']
-        self.source = ColumnDataSource({name: [] for name in names})
-
-        columns = {name: TableColumn(field=name, title=name)
-                   for name in names}
-
-        table = DataTable(
-            source=self.source, columns=[columns[n] for n in names],
-            height=70,
-        )
-        self.root = table
-
-    def update(self):
-        with log_errors():
-            s = self.scheduler
-            d = {'Tasks': [len(s.tasks)],
-                 'Stored': [len(s.who_has)],
-                 'Processing': ['%d / %d' % (len(s.rprocessing), s.total_ncores)],
-                 'Waiting': [len(s.waiting)],
-                 'No Worker': [len(s.unrunnable)],
-                 'Erred': [len(s.exceptions)],
-                 'Released': [len(s.released)]}
-
-            update(self.source, d)
-
-
 class Occupancy(DashboardComponent):
     """ Occupancy (in time) per worker """
 
@@ -157,24 +123,23 @@ class Occupancy(DashboardComponent):
 
     def update(self):
         with log_errors():
-            o = self.scheduler.occupancy
-            workers = list(self.scheduler.workers)
+            workers = list(self.scheduler.workers.values())
 
             bokeh_addresses = []
-            for worker in workers:
-                addr = self.scheduler.get_worker_service_addr(worker, 'bokeh')
+            for ws in workers:
+                addr = self.scheduler.get_worker_service_addr(ws.address, 'bokeh')
                 bokeh_addresses.append('%s:%d' % addr if addr is not None else '')
 
             y = list(range(len(workers)))
-            occupancy = [o[w] for w in workers]
+            occupancy = [ws.occupancy for ws in workers]
             ms = [occ * 1000 for occ in occupancy]
             x = [occ / 500 for occ in occupancy]
             total = sum(occupancy)
             color = []
-            for w in workers:
-                if w in self.scheduler.idle:
+            for ws in workers:
+                if ws in self.scheduler.idle:
                     color.append('red')
-                elif w in self.scheduler.saturated:
+                elif ws in self.scheduler.saturated:
                     color.append('green')
                 else:
                     color.append('blue')
@@ -188,7 +153,7 @@ class Occupancy(DashboardComponent):
 
             if occupancy:
                 result = {'occupancy': occupancy,
-                          'worker': workers,
+                          'worker': [ws.address for ws in workers],
                           'ms': ms,
                           'color': color,
                           'bokeh_address': bokeh_addresses,
@@ -223,7 +188,7 @@ class ProcessingHistogram(DashboardComponent):
                            color='blue')
 
     def update(self):
-        L = list(map(len, self.scheduler.processing.values()))
+        L = [len(ws.processing) for ws in self.scheduler.workers.values()]
         counts, x = np.histogram(L, bins=40)
         self.source.data.update({'left': x[:-1],
                                  'right': x[1:],
@@ -241,7 +206,7 @@ class NBytesHistogram(DashboardComponent):
                                             'right': [10, 10],
                                             'top': [0, 0]})
 
-            self.root = figure(title='NBytes Stored',
+            self.root = figure(title='Bytes Stored',
                                id='bk-nbytes-histogram-plot',
                                **kwargs)
             self.root.xaxis[0].formatter = NumeralTickFormatter(format='0.0 b')
@@ -258,7 +223,7 @@ class NBytesHistogram(DashboardComponent):
                            color='blue')
 
     def update(self):
-        nbytes = np.asarray(list(self.scheduler.worker_bytes.values()))
+        nbytes = np.asarray([ws.nbytes for ws in self.scheduler.workers.values()])
         counts, x = np.histogram(nbytes, bins=40)
         d = {'left': x[:-1], 'right': x[1:], 'top': counts}
         self.source.data.update(d)
@@ -325,7 +290,7 @@ class CurrentLoad(DashboardComponent):
             processing.add_tools(hover)
 
             hover = HoverTool()
-            hover.tooltips = "@worker : @nbytes_text bytes"
+            hover.tooltips = "@worker : @nbytes_text"
             hover.point_policy = 'follow_mouse'
             nbytes.add_tools(hover)
 
@@ -337,32 +302,31 @@ class CurrentLoad(DashboardComponent):
 
     def update(self):
         with log_errors():
-            processing = valmap(len, self.scheduler.processing)
-            workers = list(self.scheduler.workers)
+            workers = list(self.scheduler.workers.values())
 
             bokeh_addresses = []
-            for worker in workers:
-                addr = self.scheduler.get_worker_service_addr(worker, 'bokeh')
+            for ws in workers:
+                addr = self.scheduler.get_worker_service_addr(ws.address, 'bokeh')
                 bokeh_addresses.append('%s:%d' % addr if addr is not None else '')
 
             y = list(range(len(workers)))
-            nprocessing = [processing[w] for w in workers]
+            nprocessing = [len(ws.processing) for ws in workers]
             processing_color = []
-            for w in workers:
-                if w in self.scheduler.idle:
+            for ws in workers:
+                if ws in self.scheduler.idle:
                     processing_color.append('red')
-                elif w in self.scheduler.saturated:
+                elif ws in self.scheduler.saturated:
                     processing_color.append('green')
                 else:
                     processing_color.append('blue')
 
-            nbytes = [self.scheduler.worker_bytes[w] for w in workers]
+            nbytes = [ws.info['memory'] for ws in workers]
             nbytes_text = [format_bytes(nb) for nb in nbytes]
             nbytes_color = []
             max_limit = 0
-            for w, nb in zip(workers, nbytes):
+            for ws, nb in zip(workers, nbytes):
                 try:
-                    limit = self.scheduler.worker_info[w]['memory_limit']
+                    limit = self.scheduler.worker_info[ws.address]['memory_limit']
                 except KeyError:
                     limit = 16e9
                 if limit > max_limit:
@@ -386,7 +350,7 @@ class CurrentLoad(DashboardComponent):
                           'nbytes-color': nbytes_color,
                           'nbytes_text': nbytes_text,
                           'bokeh_address': bokeh_addresses,
-                          'worker': workers,
+                          'worker': [ws.address for ws in workers],
                           'y': y}
 
                 self.nbytes_figure.title.text = 'Bytes stored: ' + format_bytes(sum(nbytes))
@@ -411,7 +375,7 @@ class StealingTimeSeries(DashboardComponent):
         fig.yaxis.minor_tick_line_color = None
 
         fig.add_tools(
-            ResetTool(reset_size=False),
+            ResetTool(),
             PanTool(dimensions="width"),
             WheelZoomTool(dimensions="width")
         )
@@ -456,7 +420,7 @@ class StealingEvents(DashboardComponent):
 
         fig.add_tools(
             hover,
-            ResetTool(reset_size=False),
+            ResetTool(),
             PanTool(dimensions="width"),
             WheelZoomTool(dimensions="width")
         )
@@ -525,7 +489,7 @@ class Events(DashboardComponent):
 
         fig.add_tools(
             hover,
-            ResetTool(reset_size=False),
+            ResetTool(),
             PanTool(dimensions="width"),
             WheelZoomTool(dimensions="width")
         )
@@ -639,7 +603,8 @@ class TaskProgress(DashboardComponent):
         else:
             self.plugin = AllProgress(scheduler)
 
-        data = progress_quads(dict(all={}, memory={}, erred={}, released={}))
+        data = progress_quads(dict(all={}, memory={}, erred={}, released={},
+                                   processing={}))
         self.source = ColumnDataSource(data=data)
 
         x_range = DataRange1d(range_padding=0)
@@ -650,11 +615,11 @@ class TaskProgress(DashboardComponent):
             x_range=x_range, y_range=y_range, toolbar_location=None, **kwargs
         )
         self.root.line(  # just to define early ranges
-            x=[0, 1], y=[-1, 0], line_color="#FFFFFF", alpha=0.0)
+            x=[0, 0.9], y=[-1, 0], line_color="#FFFFFF", alpha=0.0)
         self.root.quad(
             source=self.source,
             top='top', bottom='bottom', left='left', right='right',
-            fill_color="#aaaaaa", line_color="#aaaaaa", fill_alpha=0.2
+            fill_color="#aaaaaa", line_color="#aaaaaa", fill_alpha=0.1
         )
         self.root.quad(
             source=self.source,
@@ -669,9 +634,15 @@ class TaskProgress(DashboardComponent):
         )
         self.root.quad(
             source=self.source,
-            top='top', bottom='bottom', left='released-loc',
-            right='erred-loc', fill_color='black', line_color='#000000',
-            fill_alpha=0.5
+            top='top', bottom='bottom', left='memory-loc',
+            right='erred-loc', fill_color='black',
+            fill_alpha=0.5, line_alpha=0,
+        )
+        self.root.quad(
+            source=self.source,
+            top='top', bottom='bottom', left='erred-loc',
+            right='processing-loc', fill_color='gray',
+            fill_alpha=0.35, line_alpha=0,
         )
         self.root.text(
             source=self.source,
@@ -709,6 +680,10 @@ class TaskProgress(DashboardComponent):
                     <span style="font-size: 14px; font-weight: bold;">Erred:</span>&nbsp;
                     <span style="font-size: 10px; font-family: Monaco, monospace;">@erred</span>
                 </div>
+                <div>
+                    <span style="font-size: 14px; font-weight: bold;">Ready:</span>&nbsp;
+                    <span style="font-size: 10px; font-family: Monaco, monospace;">@processing</span>
+                </div>
                 """
         )
         self.root.add_tools(hover)
@@ -717,7 +692,7 @@ class TaskProgress(DashboardComponent):
         with log_errors():
             state = {'all': valmap(len, self.plugin.all),
                      'nbytes': self.plugin.nbytes}
-            for k in ['memory', 'erred', 'released']:
+            for k in ['memory', 'erred', 'released', 'processing']:
                 state[k] = valmap(len, self.plugin.state[k])
             if not state['all'] and not len(self.source.data['all']):
                 return
@@ -800,37 +775,39 @@ class WorkerTable(DashboardComponent):
     plot laying out hosts by their current memory use.
     """
 
-    def __init__(self, scheduler, **kwargs):
+    def __init__(self, scheduler, width=800, **kwargs):
         self.scheduler = scheduler
-        self.names = ['disk-read', 'cores', 'cpu', 'disk-write',
-                      'memory', 'last-seen', 'memory_percent', 'host',
-                      'network-send', 'network-recv']
+        self.names = ['worker', 'ncores', 'cpu', 'memory', 'memory_limit',
+                      'memory_percent', 'num_fds', 'read_bytes', 'write_bytes',
+                      'cpu_fraction']
+
+        table_names = ['worker', 'ncores', 'cpu', 'memory', 'memory_limit',
+                       'memory_percent', 'num_fds', 'read_bytes',
+                       'write_bytes']
+
         self.source = ColumnDataSource({k: [] for k in self.names})
 
         columns = {name: TableColumn(field=name,
                                      title=name.replace('_percent', ' %'))
-                   for name in self.names}
-
-        cnames = ['host', 'cores', 'memory', 'cpu', 'memory_percent',
-                  'network-send', 'network-recv']
+                   for name in table_names}
 
         formatters = {'cpu': NumberFormatter(format='0.0 %'),
                       'memory_percent': NumberFormatter(format='0.0 %'),
                       'memory': NumberFormatter(format='0 b'),
-                      'latency': NumberFormatter(format='0.00000'),
-                      'last-seen': NumberFormatter(format='0.000'),
-                      'disk-read': NumberFormatter(format='0 b'),
-                      'disk-write': NumberFormatter(format='0 b'),
-                      'network-send': NumberFormatter(format='0 b'),
-                      'network-recv': NumberFormatter(format='0 b')}
+                      'memory_limit': NumberFormatter(format='0 b'),
+                      'read_bytes': NumberFormatter(format='0 b'),
+                      'write_bytes': NumberFormatter(format='0 b'),
+                      'num_fds': NumberFormatter(format='0'),
+                      'ncores': NumberFormatter(format='0')}
 
         table = DataTable(
-            source=self.source, columns=[columns[n] for n in cnames],
+            source=self.source, columns=[columns[n] for n in table_names],
+            row_headers=False, reorderable=True, sortable=True, width=width,
         )
 
-        for name in cnames:
+        for name in table_names:
             if name in formatters:
-                table.columns[cnames.index(name)].formatter = formatters[name]
+                table.columns[table_names.index(name)].formatter = formatters[name]
 
         hover = HoverTool(
             point_policy="follow_mouse",
@@ -843,12 +820,13 @@ class WorkerTable(DashboardComponent):
         )
 
         mem_plot = figure(title='Memory Use (%)', toolbar_location=None,
-                          x_range=(0, 1), y_range=(-0.1, 0.1), height=80,
-                          tools='', **kwargs)
+                          x_range=(0, 1), y_range=(-0.1, 0.1), height=60,
+                          width=width, tools='', **kwargs)
         mem_plot.circle(source=self.source, x='memory_percent', y=0,
                         size=10, fill_alpha=0.5)
         mem_plot.ygrid.visible = False
         mem_plot.yaxis.minor_tick_line_alpha = 0
+        mem_plot.xaxis.visible = False
         mem_plot.yaxis.visible = False
         mem_plot.add_tools(hover, BoxSelectTool())
 
@@ -856,21 +834,23 @@ class WorkerTable(DashboardComponent):
             point_policy="follow_mouse",
             tooltips="""
                 <div>
-                  <span style="font-size: 10px; font-family: Monaco, monospace;">@host: </span>
+                  <span style="font-size: 10px; font-family: Monaco, monospace;">@worker: </span>
                   <span style="font-size: 10px; font-family: Monaco, monospace;">@cpu</span>
                 </div>
                 """
         )
 
         cpu_plot = figure(title='CPU Use (%)', toolbar_location=None,
-                          x_range=(0, 1), y_range=(-0.1, 0.1), height=80,
-                          tools='', **kwargs)
-        cpu_plot.circle(source=self.source, x='cpu', y=0,
+                          x_range=(0, 1), y_range=(-0.1, 0.1), height=60,
+                          width=width, tools='', **kwargs)
+        cpu_plot.circle(source=self.source, x='cpu_fraction', y=0,
                         size=10, fill_alpha=0.5)
         cpu_plot.ygrid.visible = False
         cpu_plot.yaxis.minor_tick_line_alpha = 0
+        cpu_plot.xaxis.visible = False
         cpu_plot.yaxis.visible = False
         cpu_plot.add_tools(hover, BoxSelectTool())
+        self.cpu_plot = cpu_plot
 
         if 'sizing_mode' in kwargs:
             sizing_mode = {'sizing_mode': kwargs['sizing_mode']}
@@ -881,28 +861,27 @@ class WorkerTable(DashboardComponent):
 
     def update(self):
         data = {name: [] for name in self.names}
-        for host in sorted(self.scheduler.host_info):
-            info = self.scheduler.host_info[host]
+        for worker, info in sorted(self.scheduler.worker_info.items()):
             for name in self.names:
                 data[name].append(info.get(name, None))
-            data['host'][-1] = host
-
-        for name in ['cpu', 'memory_percent']:
-            data[name] = [x / 100 for x in data[name]]
+            data['worker'][-1] = worker
+            if info['memory_limit']:
+                data['memory_percent'][-1] = info['memory'] / info['memory_limit']
+            else:
+                data['memory_percent'][-1] = ''
+            data['cpu'][-1] = info['cpu'] / 100.0
+            data['cpu_fraction'][-1] = info['cpu'] / 100.0 / info['ncores']
 
         self.source.data.update(data)
 
 
 def systemmonitor_doc(scheduler, extra, doc):
     with log_errors():
-        table = StateTable(scheduler)
         sysmon = SystemMonitor(scheduler, sizing_mode='scale_width')
         doc.title = "Dask Scheduler Internal Monitor"
-        doc.add_periodic_callback(table.update, 500)
         doc.add_periodic_callback(sysmon.update, 500)
 
-        doc.add_root(column(table.root, sysmon.root,
-                            sizing_mode='scale_width'))
+        doc.add_root(column(sysmon.root, sizing_mode='scale_width'))
         doc.template = template
         doc.template_variables['active_page'] = 'system'
         doc.template_variables.update(extra)
@@ -910,18 +889,16 @@ def systemmonitor_doc(scheduler, extra, doc):
 
 def stealing_doc(scheduler, extra, doc):
     with log_errors():
-        table = StateTable(scheduler)
         occupancy = Occupancy(scheduler, height=200, sizing_mode='scale_width')
         stealing_ts = StealingTimeSeries(scheduler, sizing_mode='scale_width')
         stealing_events = StealingEvents(scheduler, sizing_mode='scale_width')
         stealing_events.root.x_range = stealing_ts.root.x_range
         doc.title = "Dask Workers Monitor"
-        doc.add_periodic_callback(table.update, 500)
         doc.add_periodic_callback(occupancy.update, 500)
         doc.add_periodic_callback(stealing_ts.update, 500)
         doc.add_periodic_callback(stealing_events.update, 500)
 
-        doc.add_root(column(table.root, occupancy.root, stealing_ts.root,
+        doc.add_root(column(occupancy.root, stealing_ts.root,
                             stealing_events.root,
                             sizing_mode='scale_width'))
 
@@ -1018,24 +995,23 @@ def profile_doc(scheduler, extra, doc):
 class BokehScheduler(BokehServer):
     def __init__(self, scheduler, io_loop=None, prefix='', **kwargs):
         self.scheduler = scheduler
-        self.server_kwargs = kwargs
-        self.server_kwargs['prefix'] = prefix or None
         prefix = prefix or ''
         prefix = prefix.rstrip('/')
         if prefix and not prefix.startswith('/'):
             prefix = '/' + prefix
+        self.prefix = prefix
 
-        extra = {'prefix': prefix}
-        extra.update(template_variables)
+        self.server_kwargs = kwargs
+        self.server_kwargs['prefix'] = prefix or None
 
-        systemmonitor = Application(FunctionHandler(partial(systemmonitor_doc, scheduler, extra)))
-        workers = Application(FunctionHandler(partial(workers_doc, scheduler, extra)))
-        stealing = Application(FunctionHandler(partial(stealing_doc, scheduler, extra)))
-        counters = Application(FunctionHandler(partial(counters_doc, scheduler, extra)))
-        events = Application(FunctionHandler(partial(events_doc, scheduler, extra)))
-        tasks = Application(FunctionHandler(partial(tasks_doc, scheduler, extra)))
-        status = Application(FunctionHandler(partial(status_doc, scheduler, extra)))
-        profile = Application(FunctionHandler(partial(profile_doc, scheduler, extra)))
+        systemmonitor = Application(FunctionHandler(partial(systemmonitor_doc, scheduler, self.extra)))
+        workers = Application(FunctionHandler(partial(workers_doc, scheduler, self.extra)))
+        stealing = Application(FunctionHandler(partial(stealing_doc, scheduler, self.extra)))
+        counters = Application(FunctionHandler(partial(counters_doc, scheduler, self.extra)))
+        events = Application(FunctionHandler(partial(events_doc, scheduler, self.extra)))
+        tasks = Application(FunctionHandler(partial(tasks_doc, scheduler, self.extra)))
+        status = Application(FunctionHandler(partial(status_doc, scheduler, self.extra)))
+        profile = Application(FunctionHandler(partial(profile_doc, scheduler, self.extra)))
 
         self.apps = {
             '/system': systemmonitor,
@@ -1051,6 +1027,10 @@ class BokehScheduler(BokehServer):
         self.loop = io_loop or scheduler.loop
         self.server = None
 
+    @property
+    def extra(self):
+        return merge({'prefix': self.prefix}, template_variables)
+
     @property
     def my_server(self):
         return self.scheduler
@@ -1058,5 +1038,7 @@ class BokehScheduler(BokehServer):
     def listen(self, *args, **kwargs):
         super(BokehScheduler, self).listen(*args, **kwargs)
 
-        from .scheduler_html import get_handlers
-        self.server._tornado.add_handlers(r'.*', get_handlers(self.my_server))
+        from .scheduler_html import routes
+        handlers = [(self.prefix + '/' + url, cls, {'server': self.my_server, 'extra': self.extra})
+                    for url, cls in routes]
+        self.server._tornado.add_handlers(r'.*', handlers)
