@@ -32,7 +32,7 @@ with open(os.path.join(os.path.dirname(__file__), 'template.html')) as f:
     template_source = f.read()
 
 template = jinja2.Template(template_source)
-template_variables = {'pages': ['main', 'system', 'profile', 'crossfilter', 'counters']}
+template_variables = {'pages': ['main', 'system', 'profile', 'crossfilter']}
 
 
 class StateTable(DashboardComponent):
@@ -93,7 +93,7 @@ class CommunicatingStream(DashboardComponent):
             )
             fig.add_tools(
                 hover,
-                ResetTool(reset_size=False),
+                ResetTool(),
                 PanTool(dimensions="width"),
                 WheelZoomTool(dimensions="width")
             )
@@ -165,7 +165,7 @@ class CommunicatingTimeSeries(DashboardComponent):
         fig.line(source=self.source, x='x', y='out', color='blue')
 
         fig.add_tools(
-            ResetTool(reset_size=False),
+            ResetTool(),
             PanTool(dimensions="width"),
             WheelZoomTool(dimensions="width")
         )
@@ -193,7 +193,7 @@ class ExecutingTimeSeries(DashboardComponent):
         fig.line(source=self.source, x='x', y='y')
 
         fig.add_tools(
-            ResetTool(reset_size=False),
+            ResetTool(),
             PanTool(dimensions="width"),
             WheelZoomTool(dimensions="width")
         )
@@ -312,7 +312,7 @@ class CrossFilter(DashboardComponent):
 
             fig.add_tools(
                 # self.hover,
-                ResetTool(reset_size=False),
+                ResetTool(),
                 PanTool(),
                 WheelZoomTool(),
                 BoxZoomTool(),
@@ -644,3 +644,7 @@ class BokehWorker(BokehServer):
 
         self.loop = io_loop or worker.loop
         self.server = None
+
+    @property
+    def my_server(self):
+        return self.worker
