@@ -3,18 +3,24 @@ from __future__ import print_function, division, absolute_import
 import logging
 import sys
 
+# flake8: noqa
+
 if sys.version_info[0] == 2:
     from Queue import Queue, Empty
     from io import BytesIO
     from thread import get_ident as get_thread_identity
-    reload = reload  # flake8: noqa
-    unicode = unicode  # flake8: noqa
+    from inspect import getargspec
+    from cgi import escape as html_escape
+
+    reload = reload
+    unicode = unicode
     PY2 = True
     PY3 = False
     ConnectionRefusedError = OSError
     FileExistsError = OSError
 
     import gzip
+
     def gzip_decompress(b):
         f = gzip.GzipFile(fileobj=BytesIO(b))
         result = f.read()
@@ -46,11 +52,13 @@ if sys.version_info[0] == 2:
     logging_names = logging._levelNames
 
 if sys.version_info[0] == 3:
-    from queue import Queue, Empty  # flake8: noqa
+    from queue import Queue, Empty
     from importlib import reload
     from threading import get_ident as get_thread_identity
     from importlib import invalidate_caches
     from importlib.util import cache_from_source
+    from inspect import getfullargspec as getargspec
+    from html import escape as html_escape
 
     PY2 = False
     PY3 = True
@@ -87,7 +95,7 @@ except ImportError:
     import itertools
     from weakref import ref
 
-    class finalize:
+    class finalize(object):
         """Class for finalization of weakrefable objects
 
         finalize(obj, func, *args, **kwargs) returns a callable finalizer
@@ -184,9 +192,9 @@ except ImportError:
         @classmethod
         def _select_for_exit(cls):
             # Return live finalizers marked for exit, oldest first
-            L = [(f,i) for (f,i) in cls._registry.items() if i.atexit]
-            L.sort(key=lambda item:item[1].index)
-            return [f for (f,i) in L]
+            L = [(f, i) for (f, i) in cls._registry.items() if i.atexit]
+            L.sort(key=lambda item: item[1].index)
+            return [f for (f, i) in L]
 
         @classmethod
         def _exitfunc(cls):
