@@ -13,6 +13,7 @@ from toolz import valmap, first
 from tornado import gen
 
 from distributed import Nanny, rpc, Scheduler
+from distributed.config import config
 from distributed.core import CommClosedError
 from distributed.metrics import time
 from distributed.protocol.pickle import dumps
@@ -222,7 +223,7 @@ def test_scheduler_file():
         s.start(8008)
         w = Nanny(scheduler_file=fn)
         yield w._start()
-        assert s.workers == {w.worker_address}
+        assert set(s.workers) == {w.worker_address}
         yield w._close()
         s.stop()
 
@@ -244,7 +245,7 @@ def test_nanny_timeout(c, s, a):
 
 
 @gen_cluster(ncores=[('127.0.0.1', 1)], client=True, Worker=Nanny,
-             worker_kwargs={'memory_limit': 1e9}, timeout=20)
+             worker_kwargs={'memory_limit': 1e8}, timeout=20)
 def test_nanny_terminate(c, s, a):
     from time import sleep
 
@@ -252,7 +253,7 @@ def test_nanny_terminate(c, s, a):
         L = []
         while True:
             L.append(b'0' * 5000000)
-            sleep(0.001)
+            sleep(0.01)
 
     proc = a.process.pid
     with captured_logger(logging.getLogger('distributed.nanny')) as logger:
@@ -283,3 +284,21 @@ def test_avoid_memory_monitor_if_zero_limit(c, s):
     yield c.submit(inc, 2)  # worker doesn't pause
 
     yield nanny._close()
+
+
+@gen_cluster(ncores=[], client=True)
+def test_scheduler_address_config(c, s):
+    config['scheduler-address'] = s.address
+    try:
+        nanny = Nanny(loop=s.loop)
+        yield nanny._start()
+        assert nanny.scheduler.address == s.address
+
+        start = time()
+        while not s.workers:
+            yield gen.sleep(0.1)
+            assert time() < start + 10
+
+    finally:
+        del config['scheduler-address']
+    yield nanny._close()
