@@ -17,8 +17,8 @@ from distributed.metrics import time
 from distributed.scheduler import BANDWIDTH, key_split
 from distributed.utils_test import (slowinc, slowadd, inc, gen_cluster,
                                     slowidentity)
-from distributed.utils_test import (loop, nodebug_setup_module,
-                                    nodebug_teardown_module)  # flake8: noqa
+from distributed.utils_test import (nodebug_setup_module,
+                                    nodebug_teardown_module)
 from distributed.worker import TOTAL_MEMORY
 
 import pytest
@@ -36,7 +36,6 @@ teardown_module = nodebug_teardown_module
 def test_work_stealing(c, s, a, b):
     [x] = yield c._scatter([1], workers=a.address)
     futures = c.map(slowadd, range(50), [x] * 50)
-    yield gen.sleep(0.1)
     yield wait(futures)
     assert len(a.data) > 10
     assert len(b.data) > 10
@@ -120,7 +119,7 @@ def test_eventually_steal_unknown_functions(c, s, a, b):
     assert len(b.data) >= 3
 
 
-@pytest.mark.xfail(reason='')
+@pytest.mark.skip(reason='')
 @gen_cluster(client=True, ncores=[('127.0.0.1', 1)] * 3)
 def test_steal_related_tasks(e, s, a, b, c):
     futures = e.map(slowinc, range(20), delay=0.05, workers=a.address,
@@ -224,7 +223,8 @@ def test_dont_steal_host_restrictions(c, s, a, b):
     yield future
 
     futures = c.map(slowinc, range(100), delay=0.1, workers='127.0.0.1')
-    yield gen.sleep(0.1)
+    while len(a.task_state) < 10:
+        yield gen.sleep(0.01)
     assert len(a.task_state) == 100
     assert len(b.task_state) == 0
 
@@ -242,7 +242,8 @@ def test_dont_steal_resource_restrictions(c, s, a, b):
     yield future
 
     futures = c.map(slowinc, range(100), delay=0.1, resources={'A': 1})
-    yield gen.sleep(0.1)
+    while len(a.task_state) < 10:
+        yield gen.sleep(0.01)
     assert len(a.task_state) == 100
     assert len(b.task_state) == 0
 
@@ -253,7 +254,7 @@ def test_dont_steal_resource_restrictions(c, s, a, b):
     assert len(b.task_state) == 0
 
 
-@pytest.mark.xfail(reason='no stealing of resources')
+@pytest.mark.skip(reason='no stealing of resources')
 @gen_cluster(client=True, ncores=[('127.0.0.1', 1, {'resources': {'A': 2}})],
              timeout=3)
 def test_steal_resource_restrictions(c, s, a):
@@ -368,7 +369,6 @@ def assert_balanced(inp, expected, c, s, *workers):
     steal._pc.stop()
 
     counter = itertools.count()
-    B = BANDWIDTH
     tasks = list(concat(inp))
     data_seq = itertools.count()
 
@@ -377,7 +377,12 @@ def assert_balanced(inp, expected, c, s, *workers):
         for t in sorted(ts, reverse=True):
             if t:
                 [dat] = yield c._scatter([next(data_seq)], workers=w.address)
-                s.nbytes[dat.key] = BANDWIDTH * t
+                ts = s.tasks[dat.key]
+                # Ensure scheduler state stays consistent
+                old_nbytes = ts.nbytes
+                ts.nbytes = BANDWIDTH * t
+                for ws in ts.who_has:
+                    ws.nbytes += ts.nbytes - old_nbytes
             else:
                 dat = 123
             s.task_duration[str(int(t))] = 1
@@ -389,7 +394,6 @@ def assert_balanced(inp, expected, c, s, *workers):
     while len(s.rprocessing) < len(futures):
         yield gen.sleep(0.001)
 
-
     for i in range(10):
         steal.balance()
 
@@ -474,7 +478,8 @@ def test_balance(inp, expected):
     test()
 
 
-@gen_cluster(client=True, ncores=[('127.0.0.1', 1)] * 2, Worker=Nanny)
+@gen_cluster(client=True, ncores=[('127.0.0.1', 1)] * 2, Worker=Nanny,
+             timeout=20)
 def test_restart(c, s, a, b):
     futures = c.map(slowinc, range(100), delay=0.1, workers=a.address,
                     allow_other_workers=True)
@@ -485,7 +490,7 @@ def test_restart(c, s, a, b):
     assert any(st for st in steal.stealable_all)
     assert any(x for L in steal.stealable.values() for x in L)
 
-    yield c._restart()
+    yield c.restart(timeout=10)
 
     assert not any(x for x in steal.stealable_all)
     assert not any(x for L in steal.stealable.values() for x in L)
@@ -519,7 +524,7 @@ def test_steal_twice(c, s, a, b):
 
     futures = [c.submit(slowadd, x, i, delay=0.2) for i in range(100)]
 
-    while len(s.task_state) < 100:  # tasks are all allocated
+    while len(s.tasks) < 100:  # tasks are all allocated
         yield gen.sleep(0.01)
 
     workers = [Worker(s.ip, s.port, loop=s.loop) for _ in range(20)]
@@ -546,7 +551,8 @@ def test_dont_steal_executing_tasks(c, s, a, b):
     while not a.executing:
         yield gen.sleep(0.01)
 
-    steal.move_task_request(future.key, a.address, b.address)
+    steal.move_task_request(s.tasks[future.key],
+                            s.workers[a.address], s.workers[b.address])
     yield gen.sleep(0.1)
     assert future.key in a.executing
     assert not b.executing
