@@ -5,10 +5,8 @@ from collections import Iterable, deque
 from contextlib import contextmanager
 from datetime import timedelta
 import functools
-import gc
 import json
 import logging
-import math
 import multiprocessing
 import operator
 import os
@@ -211,7 +209,8 @@ def sync(loop, func, *args, **kwargs):
     Run coroutine in loop running in separate thread.
     """
     # Tornado's PollIOLoop doesn't raise when using closed, do it ourselves
-    if isinstance(loop, PollIOLoop) and getattr(loop, '_closing', False):
+    if ((isinstance(loop, PollIOLoop) and getattr(loop, '_closing', False)) or
+        (hasattr(loop, 'asyncio_loop') and loop.asyncio_loop._closed)):
         raise RuntimeError("IOLoop is closed")
 
     timeout = kwargs.pop('callback_timeout', None)
@@ -249,7 +248,7 @@ def sync(loop, func, *args, **kwargs):
             raise gen.TimeoutError("timed out after %s s." % (timeout,))
     else:
         while not e.is_set():
-            e.wait(1000000)
+            e.wait(10)
     if error[0]:
         six.reraise(*error[0])
     else:
@@ -277,9 +276,10 @@ class LoopRunner(object):
     _lock = threading.Lock()
 
     def __init__(self, loop=None, asynchronous=False):
+        current = IOLoop.current()
         if loop is None:
             if asynchronous:
-                self._loop = IOLoop.current()
+                self._loop = current
             else:
                 # We're expecting the loop to run in another thread,
                 # avoid re-using this thread's assigned loop
@@ -639,7 +639,7 @@ def silence_logging(level, root='distributed'):
     (or keep the existing level if less verbose).
     """
     if isinstance(level, str):
-        level = logging_names[level.upper()]
+        level = getattr(logging, level.upper())
 
     for name, logger in logging.root.manager.loggerDict.items():
         if (isinstance(logger, logging.Logger)
@@ -907,7 +907,7 @@ def ensure_bytes(s):
         return s
     if isinstance(s, memoryview):
         return s.tobytes()
-    if isinstance(s, bytearray) or PY2 and isinstance(s, buffer):  # flake8: noqa
+    if isinstance(s, bytearray) or PY2 and isinstance(s, buffer):  # noqa: F821
         return bytes(s)
     if hasattr(s, 'encode'):
         return s.encode()
@@ -1069,6 +1069,67 @@ def format_bytes(n):
     return '%d B' % n
 
 
+byte_sizes = {
+        'kB': 10**3,
+        'MB': 10**6,
+        'GB': 10**9,
+        'TB': 10**12,
+        'PB': 10**15,
+        'KiB': 2**10,
+        'MiB': 2**20,
+        'GiB': 2**30,
+        'TiB': 2**40,
+        'PiB': 2**50,
+        'B': 1,
+        '': 1,
+}
+byte_sizes = {k.lower(): v for k, v in byte_sizes.items()}
+byte_sizes.update({k[0]: v for k, v in byte_sizes.items() if k and 'i' not in k})
+byte_sizes.update({k[:-1]: v for k, v in byte_sizes.items() if k and 'i' in k})
+
+
+def parse_bytes(s):
+    """ Parse byte string to numbers
+
+    >>> parse_bytes('100')
+    100
+    >>> parse_bytes('100 MB')
+    100000000
+    >>> parse_bytes('100M')
+    100000000
+    >>> parse_bytes('5kB')
+    5000
+    >>> parse_bytes('5.4 kB')
+    5400
+    >>> parse_bytes('1kiB')
+    1024
+    >>> parse_bytes('1e6')
+    1000000
+    >>> parse_bytes('1e6 kB')
+    1000000000
+    >>> parse_bytes('MB')
+    1000000
+    """
+    s = s.replace(' ', '')
+    if not s[0].isdigit():
+        s = '1' + s
+
+    for i in range(len(s) - 1, -1, -1):
+        if not s[i].isalpha():
+            break
+    index = i + 1
+
+    prefix = s[:index]
+    suffix = s[index:]
+
+    n = float(prefix)
+
+    multiplier = byte_sizes[suffix.lower()]
+
+    result = n * multiplier
+    return int(result)
+
+
 def asciitable(columns, rows):
     """Formats an ascii table for given columns and rows.
 
@@ -1092,7 +1153,7 @@ def asciitable(columns, rows):
 
 
 if PY2:
-    def nbytes(frame, _bytes_like=(bytes, bytearray, buffer)):
+    def nbytes(frame, _bytes_like=(bytes, bytearray, buffer)):  # noqa: F821
         """ Number of bytes of a frame or memoryview """
         if isinstance(frame, _bytes_like):
             return len(frame)
@@ -1169,60 +1230,30 @@ def format_time(n):
 
 class DequeHandler(logging.Handler):
     """ A logging.Handler that records records into a deque """
+    _instances = weakref.WeakSet()
+
     def __init__(self, *args, **kwargs):
         n = kwargs.pop('n', 10000)
         self.deque = deque(maxlen=n)
         super(DequeHandler, self).__init__(*args, **kwargs)
+        self._instances.add(self)
 
     def emit(self, record):
         self.deque.append(record)
 
+    def clear(self):
+        """
+        Clear internal storage.
+        """
+        self.deque.clear()
 
-class ThrottledGC(object):
-    """Wrap gc.collect to protect against excessively repeated calls.
-
-    Allows to run throttled garbage collection in the workers as a
-    countermeasure to e.g.: https://github.com/dask/zict/issues/19
-
-    collect() does nothing when repeated calls are so costly and so frequent
-    that the thread would spend more than max_in_gc_frac doing GC.
-
-    warn_if_longer is a duration in seconds (10s by default) that can be used
-    to log a warning level message whenever an actual call to gc.collect()
-    lasts too long.
-    """
-    def __init__(self, max_in_gc_frac=0.05, warn_if_longer=1, logger=None):
-        self.max_in_gc_frac = max_in_gc_frac
-        self.warn_if_longer = warn_if_longer
-        self.last_collect = time()
-        self.last_gc_duration = 0
-        self.logger = logger if logger is not None else _logger
-
-    def collect(self):
-        # In case of non-monotonicity in the clock, assume that any Python
-        # operation lasts at least 1e-6 second.
-        collect_start = time()
-        elapsed = max(collect_start - self.last_collect, 1e-6)
-        if self.last_gc_duration / elapsed < self.max_in_gc_frac:
-            self.logger.debug("Calling gc.collect(). %0.3fs elapsed since "
-                              "previous call.", elapsed)
-            gc.collect()
-            self.last_collect = collect_start
-            self.last_gc_duration = max(time() - collect_start, 1e-6)
-            if self.last_gc_duration > self.warn_if_longer:
-                self.logger.warning("gc.collect() took %0.3fs. This is usually"
-                                    " a sign that the some tasks handle too"
-                                    " many Python objects at the same time."
-                                    " Rechunking the work into smaller tasks"
-                                    " might help.",
-                                    self.last_gc_duration)
-            else:
-                self.logger.debug("gc.collect() took %0.3fs",
-                                  self.last_gc_duration)
-        else:
-            self.logger.debug("gc.collect() lasts %0.3fs but only %0.3fs "
-                              "elapsed since last call: throttling.",
-                              self.last_gc_duration, elapsed)
+    @classmethod
+    def clear_all_instances(cls):
+        """
+        Clear the internal storage of all live DequeHandlers.
+        """
+        for inst in list(cls._instances):
+            inst.clear()
 
 
 def fix_asyncio_event_loop_policy(asyncio):
@@ -1247,6 +1278,16 @@ def fix_asyncio_event_loop_policy(asyncio):
     asyncio.set_event_loop_policy(PatchedDefaultEventLoopPolicy())
 
 
+def reset_logger_locks():
+    """ Python 2's logger's locks don't survive a fork event
+
+    https://github.com/dask/distributed/issues/1491
+    """
+    for name in logging.Logger.manager.loggerDict.keys():
+        for handler in logging.getLogger(name).handlers:
+            handler.createLock()
+
+
 # Only bother if asyncio has been loaded by Tornado
 if 'asyncio' in sys.modules:
     fix_asyncio_event_loop_policy(sys.modules['asyncio'])
