@@ -1,5 +1,6 @@
 from __future__ import print_function, division, absolute_import
 
+import collections
 from contextlib import contextmanager
 from datetime import timedelta
 import functools
@@ -21,7 +22,6 @@ import textwrap
 import threading
 from time import sleep
 import uuid
-import warnings
 import weakref
 
 try:
@@ -29,7 +29,6 @@ try:
 except ImportError:
     ssl = None
 
-import psutil
 import pytest
 import six
 
@@ -39,13 +38,14 @@ from tornado import gen, queues
 from tornado.gen import TimeoutError
 from tornado.ioloop import IOLoop
 
-from .compatibility import WINDOWS, PY3
+from .compatibility import PY3
 from .config import config, initialize_logging
 from .core import connect, rpc, CommClosedError
 from .metrics import time
-from .nanny import Nanny
+from .proctitle import enable_proctitle_on_children
 from .security import Security
-from .utils import ignoring, log_errors, sync, mp_context, get_ip, get_ipv6
+from .utils import (ignoring, log_errors, mp_context, get_ip, get_ipv6,
+                    DequeHandler, reset_logger_locks)
 from .worker import Worker, TOTAL_MEMORY
 
 
@@ -87,6 +87,7 @@ def loop():
         orig_start = loop.start
         is_stopped = threading.Event()
         is_stopped.set()
+
         def start():
             is_stopped.clear()
             try:
@@ -150,14 +151,21 @@ def pristine_loop():
 @contextmanager
 def mock_ipython():
     import mock
+    from distributed._ipython_utils import remote_magic
     ip = mock.Mock()
     ip.user_ns = {}
     ip.kernel = None
 
-    def get_ip(): return ip
+    def get_ip():
+        return ip
+
     with mock.patch('IPython.get_ipython', get_ip), \
             mock.patch('distributed._ipython_utils.get_ipython', get_ip):
         yield ip
+    # cleanup remote_magic client cache
+    for kc in remote_magic._clients.values():
+        kc.stop_channels()
+    remote_magic._clients.clear()
 
 
 def nodebug(func):
@@ -275,10 +283,20 @@ def slowidentity(*args, **kwargs):
         return args
 
 
+def run_for(duration, timer=time):
+    """
+    Burn CPU for *duration* seconds.
+    """
+    deadline = timer() + duration
+    while timer() <= deadline:
+        pass
+
+
 # This dict grows at every varying() invocation
-_varying_dict = {}
+_varying_dict = collections.defaultdict(int)
 _varying_key_gen = itertools.count()
 
+
 class _ModuleSlot(object):
     def __init__(self, modname, slotname):
         self.modname = modname
@@ -299,7 +317,6 @@ def varying(items):
     # and not a copy.
     slot = _ModuleSlot(__name__, '_varying_dict')
     key = next(_varying_key_gen)
-    _varying_dict[key] = 0
 
     def func():
         dct = slot.get()
@@ -348,7 +365,7 @@ if sys.version_info >= (3, 5):
             await gen.sleep(delay)
             return x + 1
         """)
-    assert asyncinc  # flake8: noqa
+    assert asyncinc  # noqa: F821
 else:
     asyncinc = None
 
@@ -408,6 +425,7 @@ def run_scheduler(q, nputs, **kwargs):
 def run_worker(q, scheduler_q, **kwargs):
     from distributed import Worker
 
+    reset_logger_locks()
     with log_errors():
         with pristine_loop() as loop:
             scheduler_addr = scheduler_q.get()
@@ -454,6 +472,7 @@ def check_active_rpc(loop, active_rpc_timeout=1):
     # This would happen especially if a non-localhost address is used,
     # as Nanny does.
     # (*) (example: gather_from_workers())
+
     def fail():
         pytest.fail("some RPCs left active by test: %s"
                     % (sorted(set(rpc.active) - active_before)))
@@ -475,6 +494,8 @@ def cluster(nworkers=2, nanny=False, worker_kwargs={}, active_rpc_timeout=1,
     for name, level in logging_levels.items():
         logging.getLogger(name).setLevel(level)
 
+    enable_proctitle_on_children()
+
     with pristine_loop() as loop:
         with check_active_rpc(loop, active_rpc_timeout):
             if nanny:
@@ -631,9 +652,11 @@ def start_cluster(ncores, scheduler_addr, loop, security=None,
     yield [w._start(ncore[0]) for ncore, w in zip(ncores, workers)]
 
     start = time()
-    while len(s.ncores) < len(ncores):
+    while len(s.workers) < len(ncores):
         yield gen.sleep(0.01)
         if time() - start > 5:
+            yield [w._close(timeout=1) for w in workers]
+            yield s.close(fast=True)
             raise Exception("Cluster creation timeout")
     raise gen.Return((s, workers))
 
@@ -673,16 +696,19 @@ def gen_cluster(ncores=[('127.0.0.1', 1), ('127.0.0.1', 2)],
         start
         end
     """
-    for name, level in logging_levels.items():
-        logging.getLogger(name).setLevel(level)
-
-    worker_kwargs = merge({'memory_limit': TOTAL_MEMORY}, worker_kwargs)
+    worker_kwargs = merge({'memory_limit': TOTAL_MEMORY, 'death_timeout': 5},
+                          worker_kwargs)
 
     def _(func):
         if not iscoroutinefunction(func):
             func = gen.coroutine(func)
 
         def test_func():
+            # Restore default logging levels
+            # XXX use pytest hooks/fixtures instead?
+            for name, level in logging_levels.items():
+                logging.getLogger(name).setLevel(level)
+
             old_globals = _globals.copy()
             result = None
             workers = []
@@ -691,10 +717,16 @@ def gen_cluster(ncores=[('127.0.0.1', 1), ('127.0.0.1', 2)],
                 with check_active_rpc(loop, active_rpc_timeout):
                     @gen.coroutine
                     def coro():
-                        s, ws = yield start_cluster(
-                            ncores, scheduler, loop, security=security,
-                            Worker=Worker, scheduler_kwargs=scheduler_kwargs,
-                            worker_kwargs=worker_kwargs)
+                        for i in range(5):
+                            try:
+                                s, ws = yield start_cluster(
+                                    ncores, scheduler, loop, security=security,
+                                    Worker=Worker, scheduler_kwargs=scheduler_kwargs,
+                                    worker_kwargs=worker_kwargs)
+                            except Exception:
+                                logger.error("Failed to start gen_cluster, retryng")
+                            else:
+                                break
                         workers[:] = ws
                         args = [s] + workers
                         if client:
@@ -703,8 +735,8 @@ def gen_cluster(ncores=[('127.0.0.1', 1), ('127.0.0.1', 2)],
                             args = [c] + args
                         try:
                             result = yield func(*args)
-                            # for w in workers:
-                            #     assert not w._comms
+                            if s.validate:
+                                s.validate_state()
                         finally:
                             if client:
                                 yield c._close()
@@ -725,6 +757,7 @@ def gen_cluster(ncores=[('127.0.0.1', 1), ('127.0.0.1', 2)],
                         # was already removed
                         pass
                     del w.data
+            DequeHandler.clear_all_instances()
             return result
 
         return test_func
